# Main file
# Copyright (C) 2010  Sylvain Beucler
#
# This file is part of GunDuck Force IV.
# 
# GunDuck Force IV is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# GunDuck Force IV is distributed in the hope that it will be fun,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pygame import *
import sys
import math, random


LOGIC_RATE=20
LOGIC_DURATION_MS=1*1000.0/LOGIC_RATE
SCREEN_MAX_SKIPPED_FRAME=5

SCREEN_W=640
SCREEN_H=480
PLAY_W=600
PLAY_H=400

game = None
datadir = None
white = Color(255,255,255)

class GDSprite(sprite.Sprite):
    """
    Adds camera coordinates, centering, image loading,
    interpolation/tweening
    """
    def __init__(self, *groups, **kwargs):
        super(GDSprite, self).__init__(*groups)
        # graphics
        self.anim_name = kwargs.pop('anim_name')
        # position in the level
        self.x    = kwargs.pop('x', 0.0)
        self.y    = kwargs.pop('y', 0.0)
        # position of the sprite on the screen at the last logic frame
        self.prev_x = None
        self.prev_y = None
        # Depth/z-order
        if not hasattr(self, 'layer'):
            self.layer = 0
        # Damage
        self.health = 200
        self.hittable = True
        # Set image (used by draw())
        self.anim = None
        self.w = game.anims[self.anim_name]['image'].get_width()
        if game.anims[self.anim_name].get('nb_frames', 1) == 1:
            # static picture
            self.image = game.anims[self.anim_name]['image']
            self.h = self.image.get_height()
        else:
            # animation
            self.anim = game.anims[self.anim_name]
            self.h = self.anim['height']
            self.image = self.anim['image'].subsurface( \
                Rect(0, 0, self.w, self.h))
        self.frame = 0

    def global_to_camera(self, x, y):
        x -= game.camera.x
        y -= game.camera.y
        return (x,y)

    def camera_to_global(self, x, y):
        x += game.camera.x
        y += game.camera.y
        return (x,y)

    def global_to_screen(self, x, y):
        """
        Transform the sprite coordinate from the level coordinates to
        screen coordinates; pos is the (interpolated) level
        coordinates.
        """
        # Camera offset/coordinates
        (x,y) = self.global_to_camera(x, y)
        # Center
        x -= self.w / 2
        y -= self.h / 2
        # TODO: screen shaking offset
        return (x,y)

    def update(self):
        # Save previous position for interpolation
        self.prev_x = self.x
        self.prev_y = self.y

        # Progress animation
        if self.anim is not None:
            self.image = self.anim['image'].subsurface( \
                Rect(0, self.h*self.frame, self.w, self.h))
            self.frame = (self.frame + 1) % self.anim['nb_frames']

    def get_rect(self):
        """
        .rect attribute used by sprite.Group for sprite display
        """
        global game
        (x,y) = (self.x,self.y)
        if self.prev_x is not None:
            # Linear interpolation from last frame - the downside is
            # that we're a fraction of a logic frame behind
            x = self.prev_x + (x - self.prev_x) * game.interpolation
            y = self.prev_y + (y - self.prev_y) * game.interpolation
        (x,y) = self.global_to_screen(x,y)
        return Rect(x,y,0,0)
    rect = property(get_rect)

    def die(self):
        self.kill()

class Player(GDSprite):
    WEAPON_DOUBLE = 0
    WEAPON_RAILGUN = 1
    def __init__(self, *groups, **kwargs):
        kwargs['anim_name'] = 'ship'
        super(Player, self).__init__(*groups, **kwargs)
        self.dx = 0
        self.dy = 0
        self.lframe_speed = LOGIC_DURATION_MS/1000 * 400 # (px/s)
        self.firing = False
        self.fire_counter = 0
        self.fire_counter_rail = (.35, 0)
        self.fire_position = 0
        self.weapon = Player.WEAPON_DOUBLE
    def process_event(self, ev):
        if ev.type == KEYDOWN:
            if ev.key == K_LEFT:
                self.dx = -1
            elif ev.key == K_RIGHT:
                self.dx = 1
            elif ev.key == K_UP:
                self.dy = -1
            elif ev.key == K_DOWN:
                self.dy = 1
            elif ev.key == K_LCTRL:
                self.firing = True
                self.fire_counter = 1
            elif ev.key == K_LSHIFT:
                self.switch_weapon()
        elif ev.type == KEYUP:
            if ev.key == K_LEFT and self.dx == -1:
                self.dx = 0
            elif ev.key == K_RIGHT and self.dx == 1:
                self.dx = 0
            elif ev.key == K_UP:
                self.dy = 0
            elif ev.key == K_DOWN:
                self.dy = 0
            elif ev.key == K_LCTRL:
                self.firing = False

    def switch_weapon(self):
        self.weapon = (self.weapon + 1) % 2

    def update(self):
        # Position
        super(Player, self).update()
        self.x += self.dx * self.lframe_speed
        self.y += self.dy * self.lframe_speed

        # Capping
        (x,y) = self.global_to_camera(self.x, self.y)
        min_x = self.w / 2
        max_x = SCREEN_W - self.w / 2
        min_y = self.h / 2
        max_y = SCREEN_H - self.h / 2
        x = max(x, min_x)
        x = min(x, max_x)
        y = max(y, min_y)
        y = min(y, max_y)
        (self.x, self.y) = self.camera_to_global(x, y)

        # Missile
        if self.firing:
            if self.weapon == Player.WEAPON_DOUBLE:
                lframe_fire_interval = 1
                self.fire_counter += lframe_fire_interval
                if self.fire_counter >= 1:
                    self.fire_counter %= 1
                    self.fire_position = (self.fire_position + 1) % 2
                    missile = MissileSimple(game.sprites)
                    missile.x = self.x + self.w/2 + missile.w/2
                    missile.y = self.y + (self.fire_position*2-1) * 8 # double-fire
                    game.sounds['fire'].play()
            elif self.weapon == Player.WEAPON_RAILGUN:
                lframe_fire_interval = .5
                self.fire_counter += lframe_fire_interval
                if self.fire_counter >= 1:
                    self.fire_counter %= 1
                    missile = MissileSimple(game.sprites, anim_name='missile-blue')
                    missile.x = self.x + self.w/2 + missile.w/2
                    missile.y = self.y # single fire
                game.sounds['fire-small'].play()

                lframe_fire_interval = 1.0/20
                self.fire_counter_rail = [i+lframe_fire_interval for i in self.fire_counter_rail]
                for position,counter in enumerate(self.fire_counter_rail):
                    if counter >= 1:
                        missile = MissileSimple(game.sprites, anim_name='laser', direction=math.pi, strength=150)
                        missile.x = self.x - self.w/2 - missile.w/2
                        missile.y = self.y + (position*2-1) * 8 # double-fire
                        game.sounds['railgun'].play()
                self.fire_counter_rail = [i%1 for i in self.fire_counter_rail]

class Fly(GDSprite):
    def __init__(self, *groups, **kwargs):
        kwargs['anim_name'] = 'fly'
        super(Fly, self).__init__(*groups, **kwargs)
        (self.center_x, self.center_y) = (self.x, self.y)
        # The fly will make rounds around a center point
        self.radius = 0
        self.lframe_radius_delta = LOGIC_DURATION_MS/1000 * (math.pi / .75) # radian/s
        self.ia_trigger_delta = (LOGIC_DURATION_MS/1000) / .5 # s
        self.ia_trigger_counter = 0.0
    def update(self):
        super(Fly, self).update()
        self.ia_trigger_counter += self.ia_trigger_delta
        if self.ia_trigger_counter >= 1:
            self.ia_trigger_counter %= 1
            # trigo: angle = atan(opp/adj) - using atan2 to consider
            # the full [-PI,PI] interval
            direction = math.atan2(game.player.y - self.y, game.player.x - self.x)
            MissileSimple(game.sprites, anim_name='missile-ball',
                          direction=direction, lframe_speed=LOGIC_DURATION_MS/1000 * 200,
                          x=self.x, y=self.y, mtype='enemy')
        self.radius += self.lframe_radius_delta
        self.x = self.center_x + math.cos(self.radius) * 10  # px
        self.y = self.center_y + math.sin(self.radius) * 10  # px

class MissileSimple(GDSprite):
    def __init__(self, *groups, **kwargs):
        kwargs['anim_name'] = kwargs.get('anim_name', 'missile')
        self.direction    = kwargs.pop('direction', 0) # radian
        self.strength     = kwargs.pop('strength', 10)
        self.lframe_speed = kwargs.pop('lframe_speed', LOGIC_DURATION_MS/1000 * 2000)
        self.mtype         = kwargs.pop('mtype', 'player') # enemy|player
        super(MissileSimple, self).__init__(*groups, **kwargs)
    def update(self):
        super(MissileSimple, self).update()
        # Collision detection
        if self.mtype == 'enemy':
            # for player: in the main loop
            pass
        elif self.mtype == 'player':
            left   = self.prev_x - self.w*.8
            top    = self.prev_y - self.h*.8
            right  = self.x + self.w*.8
            bottom = self.y + self.h*.8
            passed = Rect(left, top, right-left, bottom-top)
            for cur_sprite in game.sprites:
                if not isinstance(cur_sprite, Fly):
                    continue
                cur_sprite_rect = Rect(cur_sprite.x - cur_sprite.w/2,
                                       cur_sprite.y - cur_sprite.h/2,
                                       cur_sprite.w,
                                       cur_sprite.h)
                if cur_sprite_rect.colliderect(passed):
                    if cur_sprite.hittable:
                        cur_sprite.health -= self.strength;
                        AnimAndDie(game.sprites,
                                   anim_name='impact',
                                   x=cur_sprite.x, y=cur_sprite.y)
                        if cur_sprite.health <= 0:
                            cur_sprite.die()
                    else:
                        # TODO: another anim color for unbreakable things
                        pass
                    self.kill()
                    return
        
        # TODO: currently update after collision detection, otherwise
        # the missile is not even shown hitting its target, due to
        # interpolation lateness
        self.x += math.cos(self.direction) * self.lframe_speed
        self.y += math.sin(self.direction) * self.lframe_speed

class AnimAndDie(GDSprite):
    def __init__(self, *groups, **kwargs):
        super(AnimAndDie, self).__init__(*groups, **kwargs)
        self.update()

    def update(self):
        super(AnimAndDie, self).update()
        if self.frame == 0:
            self.kill()


class Level:
    def __init__(self):
        self.background = game.anims['clouds300']['image']
        self.scroll = 0
        self.prev_scroll = 0
        self.lframe_speed = LOGIC_DURATION_MS/1000 * 800  # (px/s)

    def update(self):
        # Scrolling background
        self.prev_scroll = self.scroll
        self.scroll -= self.lframe_speed
        # Level position
        delta = 2
        game.camera.x += delta
        game.player.x += delta

    def draw(self, dest, to):
        # Scrolling background in mosaic
        iscroll = self.scroll + (self.scroll - self.prev_scroll) * game.interpolation
        sy = 0
        while sy < SCREEN_H:
            sx = iscroll - SCREEN_W
            while sx < SCREEN_W:
                dest.blit(self.background, (sx,sy))
                sx += game.anims['clouds300']['image'].get_width()
            sy += game.anims['clouds300']['image'].get_height()


class Game:
    MODE_GAME = 1
    MODE_PAUSE = 2

    def __init__(self):
        global game
        game = self

        # Animations
        self.anims = {
            'life'     : {},
            'clouds300': {},
            'ship'        : {},
            'fly'         : {},
            'missile'     : {},
            'missile-blue': {},
            'laser'       : {},
            'ship_death'  : {'height': 28, 'nb_frames': 6},
            'impact'      : {'height': 16, 'nb_frames': 5},
            'missile-ball': {'height': 16, 'nb_frames': 10},
            }
        for name, anim in self.anims.items():
            anim['image'] = image.load("%s/%s.png" % (datadir, name))

        # Init sounds
        self.sounds = {}
        for name in ('pause', 'fire', 'railgun', 'fire-small'):
            self.sounds[name] = mixer.Sound("%s/sfx/%s.wav" % (datadir, name))
            # This sounds quite loud on my comp
            self.sounds[name].set_volume(.5)

        # Pause
        self.pause_font = font.Font("%s/fonts/SteveHand.ttf" % datadir, 20)
        self.pause_background = None
        self.pause_ticks = 0

        # It's not useful to display more FPS than the physical
        # screen's refresh rate: either we use OpenGL's vsync
        # capabilities (if available), either the user provides the
        # refresh rate herself (e.g. 60/70/85Hz - possibly after
        # trying to auto-detect it somehow)
        # FIXME: moved to 200 because that's the only way to get a
        # refresh-rate-portable not-too-shaky scrolling.
        user_defined_refresh_rate = 200

        self.is_running = True
        self.frames = 0
        self.fps = 0
        self.play_x = 20 # may change (e.g. shaking effect)
        self.play_y = 0  # id.
        self.lives = 1

        # Active sprites
        self.sprites = sprite.LayeredUpdates()
        self.player = Player(self.sprites, x=50, y=200)
        for i in range(0,10):
            Fly(self.sprites, x=450+i*20, y=150+i*20)
        self.sprites.move_to_front(self.player)

        self.screen_framerate_manager = FramerateManager(wait=True, rate=user_defined_refresh_rate)
        self.logic_counter = FramerateManager(wait=False, rate=LOGIC_RATE) # fixed
        self.cur_level = Level()
        self.surface = display.get_surface()
        self.shake_x = 0
        self.shake_y = 0
        self.shake_count = 0
        self.last_play_dest = Rect(self.play_x, self.shake_y,
                                   PLAY_W, PLAY_H)
        # position of the camera in the current level
        self.camera = Rect(0,0,0,0)

        # Mode
        self.switch_mode_to_game()

    def switch_mode_to_game(self):
        mouse.set_visible(False)
        self.mode = Game.MODE_GAME
        mixer.music.unpause()

    def switch_mode_to_pause(self):
        self.mode = Game.MODE_PAUSE
        # Darkened game background (partially transparent blit'd on
        # black screen)
        tmp = display.get_surface().copy()
        tmp.set_alpha(200)
        self.pause_background = display.get_surface().copy()
        self.pause_background.fill((0,0,0))
        self.pause_background.blit(tmp, (0,0))
        self.sounds['pause'].play()
        mixer.music.pause()

    def logic(self):
        """
        Logic frame
        """
        def game_update_from_input():
            for ev in event.get():
                processed = False
                if ev.type == QUIT:
                    self.is_running = False
                    return
                if ev.type == KEYDOWN:
                    if ev.key == K_ESCAPE:
                        self.is_running = False
                        return
                    if ev.key == K_F12:
                        image.save(display.get_surface(), "screenshot.png")
                        return
                if self.mode == Game.MODE_GAME:
                    if ev.type == KEYDOWN:
                        if ev.key == K_RETURN:
                            self.switch_mode_to_pause()
                            processed = True
                elif self.mode == Game.MODE_PAUSE:
                    if ev.type == KEYDOWN:
                        if ev.key == K_RETURN:
                            self.switch_mode_to_game()
                            processed = True
                if not processed:
                    self.player.process_event(ev)

        def game_update_from_logic():
            if self.mode == Game.MODE_GAME:
                #self.shake_count += 1
                #if self.shake_count == 10:
                self.shake_x = random.randint(-5, 5)
                self.shake_y = random.randint(-5, 5)
                #self.shake_count = 0

                self.sprites.update()
                self.cur_level.update()

                # kill all offscreen sprites
                for cur_sprite in self.sprites:
                    (x,y) = cur_sprite.global_to_camera(cur_sprite.x, cur_sprite.y)
                    margin = 100
                    if x > SCREEN_W+margin or x < -margin:
                        cur_sprite.kill()

                # collision
                # Not using sprite.spritecollide because it requires
                # 'rect' which is reserved for graphical /
                # interpolated positions
                if self.player.hittable:
                    p_left   = self.player.x - self.player.w / 2 * .8
                    p_right  = self.player.x + self.player.w / 2 * .8
                    p_top    = self.player.y - self.player.h / 2 * .8
                    p_bottom = self.player.y + self.player.h / 2 * .8
                    collisions = []
                    for cur_sprite in self.sprites:
                        s_left   = cur_sprite.x - cur_sprite.w / 2 * .8
                        s_right  = cur_sprite.x + cur_sprite.w / 2 * .8
                        s_top    = cur_sprite.y - cur_sprite.h / 2 * .8
                        s_bottom = cur_sprite.y + cur_sprite.h / 2 * .8
                        if not (cur_sprite == self.player
                                or s_right  < p_left or s_left > p_right
                                or s_bottom < p_top  or s_top  > p_bottom):
                            collisions.append(cur_sprite)
                    for cur_sprite in collisions:
                        if isinstance(cur_sprite, MissileSimple) and cur_sprite.mtype == 'player':
                            continue
                        #print cur_sprite
                        #print "%d,%d %d,%d %d,%d,%d,%d" % (self.player.x, self.player.y, cur_sprite.x, cur_sprite.y,
                        #                                   min_x, max_x, min_y, max_y)
                        self.player.kill()
                        self.lives -= 1
                        self.player.hittable = False
                        AnimAndDie(self.sprites,
                                   anim_name='ship_death',
                                   x=self.player.x, y=self.player.y)
                        break
            elif self.mode == Game.MODE_PAUSE:
                self.pause_ticks += 1
            
        game_update_from_input()
        game_update_from_logic()


    def draw(self, interpolation):
        """
        Graphic frame
        """
        def draw_status():
            x = SCREEN_W
            for i in range(0, self.lives):
                x -= self.anims['life']['image'].get_width()
                display.get_surface().blit(self.anims['life']['image'], (x,0))
            txt = 'No weapon'
            if game.player.weapon == Player.WEAPON_DOUBLE:
                txt = "Double Fire"
            elif game.player.weapon == Player.WEAPON_RAILGUN:
                txt = "Rail Gun"
            img = font.SysFont(font.get_default_font(), 24).render(txt, False, (255,255,255), (0,0,0))
            self.surface.blit(img, (0,0))

        def draw_fps(game):
            screen_current_fps = game.screen_framerate_manager.get_current_fps()
            screen_global_fps  = game.screen_framerate_manager.get_global_fps()
            logic_current_fps  = game.logic_counter.get_current_fps()
            logic_global_fps   = game.logic_counter.get_global_fps()
            txt = "sc=%0.2f / sg=%0.2f / lc=%0.2f / lg=%0.2f" \
                % (screen_current_fps, screen_global_fps, logic_current_fps, logic_global_fps)
            # Note: generating text using a TrueType font is _SLOW_
            img = font.SysFont(font.get_default_font(), 24).render(txt, False, (255,255,255), (0,0,0))
            self.surface.blit(img, (SCREEN_W-img.get_width()-2, SCREEN_H-img.get_height()-2))

        def diff_rect(old, new):
            """
            Rectangles to blank when an object moved from 'old' to
            'new' position+size
            """
            rects = []
            min_x = old.x
            min_w = old.w
            # Start by vertical bands
            w = new.x - old.x
            if w > 0:
                rects.append(Rect(old.x, old.y, w, old.h))
                min_x += w
                min_w -= w
            w = (old.x+old.w)-(new.x+new.w)
            if w > 0:
                rects.append(Rect(new.x+new.w, old.y, w, old.h))
                min_w -= w
            # Follow with horizontal bands, trimmed horizontaly using
            # min_x/min_h to avoid overlapping rects
            h = new.y - old.y
            if h > 0:
                rects.append(Rect(min_x, old.y, min_w, h))
            h = (old.y+old.h)-(new.y+new.h)
            if h > 0:
                rects.append(Rect(min_x, new.y+new.h, min_w, h))
            return rects

        if self.mode == Game.MODE_GAME:
            play_dest = Rect(self.play_x + self.shake_x, self.play_y + self.shake_y,
                             PLAY_W, PLAY_H)
            clean_rects = diff_rect(self.last_play_dest, play_dest)
            for r in clean_rects:
                self.surface.fill((0,0,0), r)
            self.last_play_dest = play_dest
            # ugly global to pass interpolation down to the Sprite
            self.interpolation = interpolation
            self.cur_level.draw(self.surface, play_dest)
            self.sprites.draw(self.surface)
            # Game over?
            if game.lives == 0:
                txt = self.pause_font.render("GAME OVER", True, white)
                display.get_surface().blit(txt, ((SCREEN_W - txt.get_width())  / 2,
                                                 (SCREEN_H - txt.get_height()) / 2))
            # No more flies?
            elif sum([1 for s in game.sprites if isinstance(s,Fly)]) == 0:
                txt = self.pause_font.render("LEVEL COMPLETED", True, white)
                display.get_surface().blit(txt, ((SCREEN_W - txt.get_width())  / 2,
                                                 (SCREEN_H - txt.get_height()) / 2))
            
            draw_status()
            #draw_fps(self)
        elif self.mode == Game.MODE_PAUSE:
            display.get_surface().fill((0,0,0))
            display.get_surface().blit(self.pause_background, (0,0))
            txt = self.pause_font.render("PAUSE !", True, white)
            delta_x = math.cos((self.pause_ticks)/16.) * 20
            delta_y = math.sin((self.pause_ticks)/8.)  * 20
            # Center
            display.get_surface().blit(txt, ((SCREEN_W - txt.get_width())  / 2 + delta_x,
                                             (SCREEN_H - txt.get_height()) / 2 + delta_y))

        display.flip()
        #display.update((0,0,200,50))


class FramerateManager:
    """
    Precise passive wait
    Wait accurately, working around inaccurate sleep() system calls,
    and handling slow hardware, either by relying on vsync, either by
    keeping track of the effective waited time.

    Cf. SDL_gfx for the concepts.
    """
    def __init__(self, wait, rate=0):
        self.wait = wait

        if self.wait:
            if rate == 0:
                print "rate is set zero!"
                sys.exit(1)
            self.rate = rate
            self.rate_ticks = 1000 / float(rate)
            self.target_ticks = 0

        self.fps_global_start = time.get_ticks()
        self.fps_global_frames = 0
        self.fps_current_start = time.get_ticks()
        self.fps_current_frames = 0
        self.fps_current_rate = 0
        self.lastcall = time.get_ticks()

    def framerateDelay(self):
        # FPS stats
        # - global
        self.fps_global_frames += 1
        # - current
        delta_ms = (time.get_ticks() - self.fps_current_start)
        if delta_ms < 1000:
            self.fps_current_frames += 1
        else:
            self.fps_current_rate = 1000.0 * self.fps_current_frames / delta_ms
            self.fps_current_start = time.get_ticks()
            self.fps_current_frames = 0

        # If vsync is enabled, assume the system is already
        # framerate-limited, so don't wait()
        if self.wait:
            self.target_ticks += self.rate_ticks
            current_ticks = time.get_ticks()

            if current_ticks <= self.target_ticks:
                delay = int(self.target_ticks) - current_ticks
                time.wait(delay)
            else: # system is too slow, or wait()-ed too much - reset
                self.target_ticks = current_ticks

        # Return effective waited time
        thisframe_delay = time.get_ticks() - self.lastcall
        self.lastcall = time.get_ticks()
        return thisframe_delay

    def get_global_fps(self):
        """
        Get the average FPS rate for the whole game session
        """
        delta_ms = (time.get_ticks() - self.fps_global_start)
        if delta_ms == 0:
            # Typically happens when initializing
            return 0
        return 1000.0 * self.fps_global_frames / delta_ms

    def get_current_fps(self):
        """
        Get the FPS rate for the past second.
        """
        return self.fps_current_rate
        

def myinit():
    init()
    display.set_mode((SCREEN_W,SCREEN_H), DOUBLEBUF) #|FULLSCREEN
    font.init()
    mixer.init()

def myquit():
    font.quit()
    quit()


def main(datadir_arg):
    global game, datadir

    datadir = datadir_arg
    myinit()
    Game()

    screen_skipped_frame = 0
    next_logic_frame = time.get_ticks()
    # Main game look with fixed-rate logic
    # Cf. http://dev.koonsolo.com/7/dewitters-gameloop/ for the concepts
    while game.is_running:
        # Logic
        # Always computed from the same starting ticks, for precision
        next_logic_frame += LOGIC_DURATION_MS
        game.logic()

        # Display
        drawn = False
        # Refresh as much as possible (with interpolation to have
        # intermediate frames), until we reach the screen refresh rate
        # (no need to display faster than that); handle slow hardware
        # gracefuly
        while (time.get_ticks() < next_logic_frame
               or screen_skipped_frame > SCREEN_MAX_SKIPPED_FRAME):
            # Interpolation is the completion ratio toward next frame [0,1[
            interpolation = (time.get_ticks() - (next_logic_frame - LOGIC_DURATION_MS)) / LOGIC_DURATION_MS
            game.draw(interpolation)
            drawn = True
            # Waits (if necessary) and keep track of FPS
            game.screen_framerate_manager.framerateDelay()
        if not drawn:
            screen_skipped_frame += 1

        # FPS accountancy for game logic (delay/wait disabled)
        game.logic_counter.framerateDelay()

    print game.screen_framerate_manager.get_global_fps()
    myquit()

if __name__ == "__main__":
    import os
    main(datadir_arg=os.path.join('share', 'gunduck'))
